from .base import _Parser
from .components import (
    InterestingFinding,
    WordPressVersion,
    Plugin,
    MainTheme,
    Theme,
    Timthumb,
    DBExport,
    User,
    Media,
    ConfigBackup,
    VulnAPI,
    PasswordAttack,
    NotFullyConfigured,
    Banner,
    ScanStarted,
    ScanFinished,
)
from .components.finding import _CoreFinding
from .results import _WPScanResults, _WPScanResultsSummaryRow

#################### JSON PARSER ######################


class WPScanJsonParser(_Parser):
    """Main interface to parse WPScan JSON data

    - data: The JSON structure of the WPScan output.
    - false_positives_strings: List of false positive strings.
    - show_all_details: Boolean, enable to show all wpscan infos (found by, confidence, etc).

    Once instanciated, the following properties are accessible:

    - `version`
    - `main_theme`
    - `plugins` (list)
    - `themes` (list)
    - `interesting_findings` (list)
    - `timthumbs` (list)
    - `db_exports` (list)
    - `users` (list)
    - `medias` (list)
    - `config_backups` (list)
    - `password_attack`
    - `not_fully_configured`
    - `vuln_api`
    - `banner`
    - `scan_started`
    - `scan_finished`

    All objects implements `get_alerts()`, `get_warnings()` and `get_infos()`
    """

    def __init__(self, data, false_positives_strings=None, show_all_details=False):

        # _Parser config: false positives string and verbosity (not available with cli parser)
        parser_config = dict(
            false_positives_strings=false_positives_strings,
            show_all_details=show_all_details,
        )
        super().__init__(data, **parser_config)
        self.components = []
        # Add WordPressVersion
        if "version" in self.data:
            self.version = WordPressVersion(self.data.get("version"), **parser_config)
        else:
            self.version = None
        # Add MainTheme
        if "main_theme" in self.data:
            self.main_theme = MainTheme(self.data.get("main_theme"), **parser_config)
        else:
            self.main_theme = None
        # Add Plugins
        if "plugins" in self.data:
            self.plugins = [
                Plugin(self.data.get("plugins").get(slug), **parser_config)
                for slug in self.data.get("plugins")
            ]
        else:
            self.plugins = []
        # Add Themes ; Make sure the main theme is not displayed twice
        if "themes" in self.data:
            self.themes = [
                Theme(self.data.get("themes").get(slug), **parser_config)
                for slug in self.data.get("themes")
                if not self.main_theme or slug != self.main_theme.slug
            ]
        else:
            self.themes = []
        # Add Interesting findings
        if "interesting_findings" in self.data:
            self.interesting_findings = [
                InterestingFinding(finding, **parser_config)
                for finding in self.data.get("interesting_findings")
            ]
        else:
            self.interesting_findings = []
        # Add Timthumbs
        if "timthumbs" in self.data:
            self.timthumbs = [
                Timthumb(url, self.data.get("timthumbs").get(url), **parser_config)
                for url in self.data.get("timthumbs")
            ]
        else:
            self.timthumbs = []
        # Add DBExport
        if "db_exports" in self.data:
            self.db_exports = [
                DBExport(url, self.data.get("db_exports").get(url), **parser_config)
                for url in self.data.get("db_exports")
            ]
        else:
            self.db_exports = []
        # Add Users
        if "users" in self.data:
            self.users = [
                User(url, self.data.get("users").get(url), **parser_config)
                for url in self.data.get("users")
            ]
        else:
            self.users = []
        # Add Medias
        if "medias" in self.data:
            self.medias = [
                Media(url, self.data.get("medias").get(url), **parser_config)
                for url in self.data.get("medias")
            ]
        else:
            self.medias = []
        # Add Config backups
        if "config_backups" in self.data:
            self.config_backups = [
                ConfigBackup(
                    url, self.data.get("config_backups").get(url), **parser_config
                )
                for url in self.data.get("config_backups")
            ]
        else:
            self.config_backups = []
        # Add VulnAPI
        if "vuln_api" in self.data:
            self.vuln_api = VulnAPI(self.data.get("vuln_api"), **parser_config)
        else:
            self.vuln_api = None
        # Add Password attack
        if self.data.get("password_attack", None):
            self.password_attack = PasswordAttack(
                self.data.get("password_attack"), **parser_config
            )
        else:
            self.password_attack = None
        # Add Not fully configured
        if self.data.get("not_fully_configured", None):
            self.not_fully_configured = NotFullyConfigured(
                self.data.get("not_fully_configured"), **parser_config
            )
        else:
            self.not_fully_configured = None
        # Add
        if self.data.get("banner", None):
            self.banner = Banner(self.data.get("banner"), **parser_config)
        else:
            self.banner = None
        # Add ScanStarted
        if "target_url" in self.data:
            self.scan_started = ScanStarted(self.data, **parser_config)
        else:
            self.scan_started = None
        # Add ScanFinished
        if "elapsed" in self.data:
            self.scan_finished = ScanFinished(self.data, **parser_config)
        else:
            self.scan_finished = None
        # Add Scan aborted error
        if self.data.get("scan_aborted", None):
            self.error = "Scan Aborted: {}".format(self.data["scan_aborted"])
        else:
            self.error = None
        # All all components to list
        self.components = [
            c
            for c in [self.version, self.main_theme]
            + self.plugins
            + self.themes
            + self.interesting_findings
            + [self.password_attack, self.not_fully_configured]
            + self.timthumbs
            + self.db_exports
            + self.users
            + self.medias
            + self.config_backups
            + [self.vuln_api, self.banner, self.scan_started, self.scan_finished]
            if c
        ]

    def get_infos(self):
        """Get all infos from all components and add false positives as infos with "[False positive]" prefix"""
        infos = []
        for component in self.components:
            infos.extend(component.get_infos())

            # If all vulns are ignored, add component message to infos
            component_warnings = [
                warning
                for warning in component.get_warnings()
                if not self.is_false_positive(warning)
            ]
            # Automatically add wp item infos if all vuln are ignored and component does not present another issue
            if (
                len(component_warnings) == 1
                and "The version could not be determined" in component_warnings[0]
                and not "Directory listing is enabled" in component_warnings[0]
                and not "An error log file has been found" in component_warnings[0]
            ):
                infos.extend(component_warnings)

            for alert in component.get_alerts() + component.get_warnings():
                if self.is_false_positive(alert):
                    infos.append("[False positive]\n" + alert)

        return infos

    def get_warnings(self):
        """Get all warnings from all components and igore false positives and automatically remove special warning if all vuln are ignored"""
        warnings = []
        for component in self.components:
            # Ignore false positives warnings
            component_warnings = [
                warning
                for warning in component.get_warnings()
                if not self.is_false_positive(warning)
            ]
            # Automatically remove wp item warning if all vuln are ignored and component does not present another issue
            if (
                len(component_warnings) == 1
                and "The version could not be determined" in component_warnings[0]
                and not "Directory listing is enabled" in component_warnings[0]
                and not "An error log file has been found" in component_warnings[0]
            ):
                component_warnings = []

            warnings.extend(component_warnings)

        return warnings

    def get_alerts(self):
        """Get all alerts from all components and igore false positives"""
        alerts = []
        for component in self.components:
            alerts.extend(
                [
                    alert
                    for alert in component.get_alerts()
                    if not self.is_false_positive(alert)
                ]
            )
        return alerts

    def get_results(self):
        results = _WPScanResults()
        results["infos"] = self.get_infos()
        results["warnings"] = self.get_warnings()
        results["alerts"] = self.get_alerts()
        results["summary"]["table"] = self.get_summary_list()
        results["summary"]["line"] = self.get_summary_line()
        results["error"] = self.get_error()
        return dict(results)

    def get_core_findings(self):
        """ Get only core findings. Core findings appears in the table summary.  """
        core = []
        for component in self.components:
            if isinstance(component, _CoreFinding):
                core.append(component)
        return core

    def get_summary_list(self):
        """Return a list of dict with all plugins, vuls, and statuses.  """
        summary_table = []
        for component in self.get_core_findings():
            row = _WPScanResultsSummaryRow(
                {
                    "Component": component.get_name(),
                    "Version": component.get_version(),
                    "Version State": component.get_version_status(),
                    "Vulnerabilities": component.get_vulnerabilities_string(),
                    "Status": component.get_status(),
                }
            )
            summary_table.append(dict(row))
        return summary_table

    def get_error(self):
        if self.error:
            return "WPScan failed: {}".format(self.error)
        else:
            return None



from .interesting_finding import InterestingFinding
from .banner import Banner
from .config_backup import ConfigBackup
from .db_export import DBExport
from .main_theme import MainTheme
from .media import Media
from .not_fully_configured import NotFullyConfigured
from .password_attack import PasswordAttack
from .plugin import Plugin
from .scan_finished import ScanFinished
from .scan_started import ScanStarted
from .theme import Theme
from .timthumb import Timthumb
from .user import User
from .vuln_api import VulnAPI
from .wordpress_version import WordPressVersion

__all__ = [
    "WordPressVersion",
    "Plugin",
    "Theme",
    "InterestingFinding",
    "Banner",
    "ConfigBackup",
    "DBExport",
    "MainTheme",
    "Media",
    "NotFullyConfigured",
    "PasswordAttack",
    "ScanFinished",
    "ScanStarted",
    "Timthumb",
    "User",
    "VulnAPI",
]



from .finding import _CoreFindingNoVersion


class NotFullyConfigured(_CoreFindingNoVersion):
    def __init__(self, data, *args, **kwargs):
        """From https://github.com/wpscanteam/wpscan/blob/master/app/views/json/core/not_fully_configured.erb"""

        super().__init__(data, *args, **kwargs)
        self.not_fully_configured = self.data

    def get_alerts(self):
        """Return 1 alert"""
        return ["Wordpress: {}".format(self.not_fully_configured)]

    def get_warnings(self):
        """Return empty list"""
        return []

    def get_infos(self):
        """Return empty list"""
        return []

    def get_name(self):
        return "Not fully configured"



"""
WPScan Output Parser Python library documentation.  
"""

__all__ = [
    "WPScanJsonParser",
    "WPScanCliParser",
    "parse_results_from_string",
    "parse_results_from_file",
    "format_results",
]

VERSION = "1.8.0"

###################### IMPORTS  ######################

from .parser.json_parser import WPScanJsonParser
from .parser.cli_parser import WPScanCliParser
from .parser import parse_results_from_string, parse_results_from_file
from .formatter import format_results



import collections

############ RESULTS CLASS ###########################

TEMPLATE_SCAN_RESULTS = {
    "infos": None,
    "warnings": None,
    "alerts": None,
    "summary": {"table": None, "line": None},
    "error": None,
}

TEMPLATE_SCAN_RESULTS_SUMMARY_ROW = {
    "Component": None,
    "Version": None,
    "Version State": None,
    "Vulnerabilities": None,
    "Status": None,
}


class _WPScanResults(collections.UserDict):
    def __init__(self, data=None):
        super().__init__(data)
        # Init dict with default values if not already passed with data
        for key in TEMPLATE_SCAN_RESULTS.keys():
            if key not in self.data:
                self.data[key] = TEMPLATE_SCAN_RESULTS[key]


class _WPScanResultsSummaryRow(collections.UserDict):
    def __init__(self, data=None):
        super().__init__(data)
        # Init dict with default values if not already passed with data
        for key in TEMPLATE_SCAN_RESULTS_SUMMARY_ROW.keys():
            if key not in self.data:
                self.data[key] = TEMPLATE_SCAN_RESULTS_SUMMARY_ROW[key]



import argparse
import json
import copy
import traceback
from . import VERSION
from .formatter import format_results
from .parser import parse_results_from_file
from .parser.results import _WPScanResults


class WPScanOutParseCLI:
    def __init__(self):
        """Load all config values as object properties"""

        exit_code = 0
        self.__dict__.update(vars(self.parse_args()))

        if self.version:
            self.print_version()
            exit(0)

        # Throw config errors
        if not self.wpscan_output_file:
            results = _WPScanResults()
            results["error"] = "Please provide WPScan output file to parse."
            print(format_results(results, format=self.format))
            exit(1)

        if self.summary and self.no_summary:
            results = _WPScanResults()
            results["error"] = "Incompatible options --summary and --no_summary"
            print(format_results(results, format=self.format))
            exit(1)

        if self.inline and self.format != "cli":
            results = _WPScanResults()
            results[
                "error"
            ] = "Incompatible options --inline and --format {}. You must use 'cli' format".format(
                self.format
            )
            print(format_results(results, format=self.format))
            exit(1)

        # Call parse_results_from_file()
        try:
            results = parse_results_from_file(
                self.wpscan_output_file, self.false_positive, self.show_all
            )
        except Exception:
            results = _WPScanResults()
            results["error"] = traceback.format_exc()
            print(format_results(results, format=self.format))
            exit(1)

        # Exit code determination based on the number of alerts, warnings and error
        if len(results["alerts"]) > 0:
            exit_code = 5
        elif len(results["warnings"]) > 0:
            exit_code = 6
        if results["error"]:
            if "WPScan failed" in results["error"]:
                exit_code = 4
            else:
                exit_code = 1

        # Delete infos and warnings if asked
        if self.no_infos:
            results["infos"] = None
        if self.no_warnings:
            results["infos"] = None
            results["warnings"] = None

        if not self.inline:

            if self.summary:
                results["alerts"] = None
                results["warnings"] = None
                results["infos"] = None
            if self.no_summary:
                results["summary"] = None

            # Print infos if any
            output = format_results(results, format=self.format, nocolor=self.no_color)
            if output:
                print(output)
        else:
            # Print only line
            print(results["summary"]["line"])

        exit(exit_code)

    @staticmethod
    def parse_args():
        parser = argparse.ArgumentParser(
            description="""wpscan_out_parse is a Python parser for WPScan output files (JSON and CLI).  
It analyze vulnerabilities, miscellaneous alerts and warnings and other findings.""",
            prog="wpscan_out_parse",
            formatter_class=argparse.RawDescriptionHelpFormatter,
            usage="python3 -m wpscan_out_parse [Options] <File path>",
        )

        parser.add_argument(
            "wpscan_output_file",
            help="WPScan output file to parse. ",
            metavar="<File path>",
            type=str,
            nargs="?",
        )
        parser.add_argument(
            "--format",
            metavar="<Format>",
            help='output format, choices are: "cli", "html", "json"',
            choices=["cli", "html", "json"],
            default="cli",
        )
        parser.add_argument(
            "--summary",
            action="store_true",
            help="display ony the summary of issues per component. ",
        )
        parser.add_argument(
            "--inline",
            action="store_true",
            help='display only one line like: "WPScan result summary: alerts={}, warnings={}, infos={}, error={}". ',
        )
        parser.add_argument(
            "--no_warnings",
            action="store_true",
            help="do not display warnings, only summary and alerts. Implies --no_infos. ",
        )
        parser.add_argument(
            "--no_infos",
            action="store_true",
            help="do not display informations and findinds. ",
        )
        parser.add_argument(
            "--no_summary",
            action="store_true",
            help="do not display the summary of issues. ",
        )
        parser.add_argument(
            "--show_all",
            action="store_true",
            help="show all findings details (found by, confidence, confirmed by). ",
        )
        parser.add_argument(
            "--false_positive",
            metavar="String",
            help='consider all matching messages as infos and add "[False positive]" prefix. ',
            nargs="+",
            default=None,
        )
        parser.add_argument(
            "--no_color",
            action="store_true",
            help="do not colorize output. ",
        )
        parser.add_argument(
            "--version",
            action="store_true",
            help="print wpscan_out_parse version and exit. ",
        )
        return parser.parse_args()

    def print_version(self):
        print("wpscan_out_parse version {}".format(VERSION))


def main():
    WPScanOutParseCLI()


if __name__ == "__main__":
    main()



import json

from .json_parser import WPScanJsonParser
from .cli_parser import WPScanCliParser

####################### INTERFACE METHODS ####################


def parse_results_from_string(
    wpscan_output_string, false_positives_strings=None, show_all_details=False
):
    """Parse any WPScan output string.

    - wpscan_output_string: WPScan output as string
    - false_positives_strings: List of false positive strings.
    - show_all_details: Boolean, enable to show all wpscan infos (found by, confidence, etc). Only with JSON output.

    Return the results as dict object"""
    try:
        data = json.loads(wpscan_output_string)
    except ValueError:
        parser = WPScanCliParser(wpscan_output_string, false_positives_strings)
    else:
        parser = WPScanJsonParser(data, false_positives_strings, show_all_details)
    return parser.get_results()


def parse_results_from_file(
    wpscan_output_file, false_positives_strings=None, show_all_details=False
):
    """Parse any WPScan output file.

    - wpscan_output_file: Path to WPScan output file
    - false_positives_strings: List of false positive strings.
    - show_all_details: Boolean, enable to show all wpscan infos (found by, confidence, etc). Only with JSON output.

     Return the results as dict object"""
    with open(wpscan_output_file, "r", encoding="utf-8") as wpscan_out:
        wpscan_out_string = wpscan_out.read()
        results = parse_results_from_string(
            wpscan_out_string,
            false_positives_strings=false_positives_strings,
            show_all_details=show_all_details,
        )

    return results



from .finding import _Finding


class WPItemVersion(_Finding):
    def __init__(self, data, *args, **kwargs):
        """Themes, plugins and timthumbs Version. From:
        https://github.com/wpscanteam/wpscan/blob/master/app/views/json/theme.erb
        https://github.com/wpscanteam/wpscan/blob/master/app/views/json/enumeration/plugins.erb
        https://github.com/wpscanteam/wpscan/blob/master/app/views/json/enumeration/timthumbs.erb
        """

        super().__init__(data, *args, **kwargs)
        self.number = self.data.get("number", None)

    def get_alerts(self):
        """Return any item version vulnerabilities"""
        return super().get_alerts()

    def get_warnings(self):
        """Return empty list"""
        return []

    def get_infos(self):
        """Return 0 or 1 info. No infos if version could not be recognized"""
        if self.number:
            info = "Version: {}".format(self.number)
            # If finding infos are present, add them
            if super().get_infos()[0] and self.show_all_details:
                info += "\n{}".format(super().get_infos()[0])
            return [info]
        else:
            return []

    def get_version(self):
        if self.get_infos():
            return self.number
        else:
            return "Unknown"



from .wp_item import WPItem


class Timthumb(WPItem):
    def __init__(self, url, data, *args, **kwargs):
        """From https://github.com/wpscanteam/wpscan/blob/master/app/views/json/enumeration/timthumbs.erb"""

        super().__init__(data, *args, **kwargs)
        self.url = url

    def get_infos(self):
        """Return 0 or 1 info, no info if Timthumb triggered warning, use get_warnings()"""
        return [
            "Timthumb: {}\n{}".format(self.url, info) for info in super().get_infos()
        ]

    def get_warnings(self):
        """Returns warnings"""
        return [
            "Timthumb: {}\n{}".format(self.url, alert)
            for alert in super().get_warnings() + self.version.get_alerts()
        ]

    def get_alerts(self):
        """Return timthumb vulnerabilities"""
        return [
            "Timthumb: {}\n{}".format(self.url, alert)
            for alert in super().get_alerts() + self.version.get_alerts()
        ]

    def get_name(self):
        return "Timthumb"



from ..base import _Component


class VulnAPI(_Component):
    def __init__(self, data, *args, **kwargs):
        """From https://github.com/wpscanteam/wpscan/blob/master/app/views/json/vuln_api/status.erb"""

        super().__init__(data, *args, **kwargs)

        self.http_error = self.data.get("http_error", None)
        self.error = self.data.get("error", None)

        self.plan = self.data.get("plan", None)
        self.requests_done_during_scan = self.data.get(
            "requests_done_during_scan", None
        )
        self.requests_remaining = self.data.get("requests_remaining", None)

    def get_infos(self):
        """Return 1 WPVulnDB info"""
        info = "WPVulnDB API"
        info += "\nPlan: {}".format(self.plan)
        info += "\nRequests: {} done during scan, {} remaining".format(
            self.requests_done_during_scan, self.requests_remaining
        )
        return [info]

    def get_warnings(self):
        """Return 0 or 1 warning. VulnAPI error No WPVulnDB API Token given or HTTP errors"""
        warning = ""
        if self.http_error:
            warning += "HTTP Error: {}".format(self.http_error)
        if self.error:
            warning += self.error
        if warning:
            return [warning]
        else:
            return []

    def get_alerts(self):
        """Return empty list"""
        return []



import unittest
from wpscan_out_parse import parse_results_from_string, parse_results_from_file

class T(unittest.TestCase):
    
       def test_parser(self):
              # false positives
              out = open("test/output_files/wordpress_no_vuln.json").read()
              result=parse_results_from_string(out)
              self.assertEqual(0, len(result['alerts']))

              out = open("test/output_files/wordpress_one_vuln.json").read()
              result=parse_results_from_string(out)
              self.assertEqual(4, len(result['warnings']))

              out = open("test/output_files/wordpress_many_vuln.json").read()
              result=parse_results_from_string(out)
              self.assertEqual(1, len(result['alerts']))

              out = open("test/output_files/wordpress_no_vuln.txt").read()
              result=parse_results_from_string(out)
              self.assertEqual(0, len(result['alerts']))

              out = open("test/output_files/wordpress_one_warning.txt").read()
              result=parse_results_from_string(out)
              self.assertEqual(2, len(result['warnings']))

              out = open("test/output_files/wordpress_many_vuln.txt").read()
              result=parse_results_from_string(out)
              self.assertEqual(8, len(result['alerts']))
              
              out = open("test/output_files/wordpress_one_vuln.txt").read()
              result=parse_results_from_string(out)
              self.assertEqual(1, len(result['alerts']))

       def test_false_positives(self):

              result=parse_results_from_file("test/output_files/wordpress_many_vuln.txt", false_positives_strings=
                     ["Yoast SEO 1.2.0-11.5 - Authenticated Stored XSS", 
                     "Yoast SEO <= 9.1 - Authenticated Race Condition"])

              self.assertEqual(6, len(result['alerts']))

       def test_version_could_not_be_detected_potential_vulns_warnings(self): 
              result=parse_results_from_file("test/output_files/potential_vulns.json")
              result2=parse_results_from_file("test/output_files/potential_vulns.json", 
                     false_positives_strings=["Potential Vulnerability"])

              self.assertEqual(len(result2['warnings'])+2, len(result['warnings']))

       # def test_oudated_plugin_or_theme_version_warning(self):
       #        pass

       # def test_vulnerabilities(self):
       #        pass

       # def test_insecure_wordpress_warning(self):
       #        pass

       # def test_password_attack(self):
       #        pass

       # def test_lots_of_enumeration(self):
       #        pass

       # def test_ref_metasploit_cve_exploitdb(self):
       #        pass


from .finding import _CoreFindingNoVersion
from .user import User


class PasswordAttack(_CoreFindingNoVersion):
    def __init__(self, data, *args, **kwargs):
        """From https://github.com/wpscanteam/wpscan/blob/master/app/views/json/password_attack/users.erb"""

        super().__init__(data, *args, **kwargs)

        self.users = [User(user, self.data.get(user), *args, **kwargs) for user in data]

    def get_alerts(self):
        """Return Password Attack Valid Combinations Found alerts"""
        alerts = []
        for user in self.users:
            alert = "Password Attack Valid Combinations Found:"
            if user.get_alerts():
                alert += "\n{}".format(user.get_alerts()[0])
                alerts.append(alert)

        return alerts

    def get_warnings(self):
        """Return empty list"""
        return []

    def get_infos(self):
        """Return empty list"""
        return []

    def get_name(self):
        return "Password Attack: {} found".format(len(self.get_alerts()))



from ..base import _Component


class ScanStarted(_Component):
    def __init__(self, data, *args, **kwargs):
        """From https://github.com/wpscanteam/CMSScanner/blob/master/app/views/json/core/started.erb"""

        super().__init__(data, *args, **kwargs)

        self.start_time = self.data.get("start_time", None)
        self.start_memory = self.data.get("start_memory", None)
        self.target_url = self.data.get("target_url", None)
        self.target_ip = self.data.get("target_ip", None)
        self.effective_url = self.data.get("effective_url", None)

    def get_infos(self):
        """Return 1 Scan Scanned info"""

        info = "Target URL: {}".format(self.target_url)
        info += "\nTarget IP: {}".format(self.target_ip)
        info += "\nEffective URL: {}".format(self.effective_url)
        if self.show_all_details:
            info += "\nStart Time: {}".format(self.start_time)
            info += "\nStart Memory: {}".format(self.start_memory)

        return [info]

    def get_warnings(self):
        """Return empty list"""
        return []

    def get_alerts(self):
        """Return empty list"""
        return []



#! /usr/bin/env python3
from setuptools import setup, find_packages
import sys
if sys.version_info[0] < 3: 
    raise RuntimeError("Sorry, you must use Python 3")
# The directory containing this file
import pathlib
HERE = pathlib.Path(__file__).parent
# Helper method that will parse wpscan_out_parse/__init__.py to extract config setup values
def parse_setup(key):
    part={}
    for line in WPWATCHER.splitlines():
        if key in line:
            exec(line, part)
            break
    return(part[key])
# Read and store wpscan_out_parse/__init__.py file
WPWATCHER = (HERE / "wpscan_out_parse" / "__init__.py").read_text()
# The text of the README file
README = (HERE / "README.md").read_text()
setup(
    name                =   'wpscan_out_parse',
    description         =   "wpscan_out_parse is a Python parser for WPScan output files (JSON and CLI). It analyze vulnerabilities, miscellaneous alerts and warnings and other findings.",
    url                 =   'http://github.com/tristanlatr/wpscan_out_parse',
    maintainer          =   'tristanlatr',
    version             =   parse_setup('VERSION'),
    packages            =   find_packages(exclude=('test')), 
    entry_points        =   {'console_scripts': ['wpscan_out_parse = wpscan_out_parse.__main__:main'],},
    classifiers         =   ["Programming Language :: Python :: 3"],
    license             =   'MIT',
    long_description    =   README,
    long_description_content_type   =   "text/markdown",
    install_requires    =   ['ansicolors']
)

# Generating docs with python3 -m pdoc wpscan_out_parse --html --force -o docs


from ..base import _Component


class Banner(_Component):
    def __init__(self, data, *args, **kwargs):
        """From https://github.com/wpscanteam/wpscan/blob/master/app/views/json/core/banner.erb"""

        super().__init__(data, *args, **kwargs)

        self.description = self.data.get("description", None)
        self.version = self.data.get("version", None)
        self.authors = self.data.get("authors", None)
        self.sponsor = self.data.get("sponsor", None) or self.data.get(
            "sponsored_by", None
        )

    def get_infos(self):
        info = "Scanned with {}".format(self.description)
        info += "\nVersion: {}".format(self.version)
        if self.show_all_details:
            info += "\nAuthors: {}".format(", ".join(self.authors))
            if self.sponsor:
                info += "\nSponsor: {}".format(self.sponsor)

        return [info]

    def get_warnings(self):
        """Return empty list"""
        return []

    def get_alerts(self):
        """Return empty list"""
        return []



from abc import ABC, abstractmethod

########### BASE CLASS FOR CLI AND JSON PARSERS ##########


class _Component(ABC):
    """Base abstract class for all WPScan JSON and CLI components"""

    def __init__(self, data, false_positives_strings, show_all_details):

        if not data:
            data = {}
        self.data = data
        self.false_positives_strings = (
            false_positives_strings if false_positives_strings else []
        )
        self.show_all_details = False
        if show_all_details:
            self.show_all_details = True

    def is_false_positive(self, string):
        """False Positive Detection"""
        if not self.false_positives_strings:
            return False
        for fp_string in self.false_positives_strings:
            if fp_string in string:
                return True
        return False

    @abstractmethod
    def get_infos(self):
        """Return the component informations as a list of strings.  """
        pass

    @abstractmethod
    def get_warnings(self):
        """Return the component warnings as a list of strings.  """
        pass

    @abstractmethod
    def get_alerts(self):
        """Return the component alerts as a list of strings.  """
        pass


class _Parser(_Component):
    """Common class for CLI and JSON parsers.  """

    def __init__(self, data, *args, **kwargs):

        super().__init__(data, *args, **kwargs)

    @abstractmethod
    def get_results(self):
        """Returns a dictionnary structure like:
        ```
        {
        'infos':[],
        'warnings':[],
        'alerts':[],
        'summary':{
            'table':[
                {
                    'Component': None,
                    'Version': None,
                    'Version State': None,
                    'Vulnerabilities': None,
                    'Status': None
                },
                ...
            ],
            'line':'WPScan result summary: alerts={}, warnings={}, infos={}, error={}'
            },
        'error':None
        }
        ```
        """
        pass

    def get_summary_line(self):
        """Return the summary string in one line"""
        line = (
            "WPScan result summary: alerts={}, warnings={}, infos={}, error={}".format(
                len(self.get_alerts()),
                len(self.get_warnings()),
                len(self.get_infos()),
                "1" if self.get_error() else "0",
            )
        )
        return line

    @abstractmethod
    def get_error(self):
        """ Return any error or None if no errors"""
        pass



from .wp_item import WPItem


class Theme(WPItem):
    def __init__(self, data, *args, **kwargs):
        """From https://github.com/wpscanteam/wpscan/blob/master/app/views/json/theme.erb"""

        super().__init__(data, *args, **kwargs)

        self.style_url = self.data.get("style_url", None)
        self.style_name = self.data.get("style_name", None)
        self.style_uri = self.data.get("style_uri", None)
        self.description = self.data.get("description", None)
        self.author = self.data.get("author", None)
        self.author_uri = self.data.get("author_uri", None)
        self.template = self.data.get("template", None)
        self.license = self.data.get("license", None)
        self.license_uri = self.data.get("license_uri", None)
        self.tags = self.data.get("tags", None)
        self.text_domain = self.data.get("text_domain", None)
        self.parents = [
            Theme(theme, *args, **kwargs) for theme in self.data.get("parents", [])
        ]

    def _get_infos(self):
        """Return 1 info"""
        info = super()._get_infos()[0]

        if self.style_url:
            info += "\nStyle CSS: {}".format(self.style_url)
        if self.style_name and self.show_all_details:
            info += "\nStyle Name: {}".format(self.style_name)
        if self.style_uri:
            info += "\nStyle URI: {}".format(self.style_uri)
        if self.description and self.show_all_details:
            info += "\nDescription: {}".format(self.description)
        if self.author:
            info += "\nAuthor: {}".format(self.author)
            if self.author_uri:
                info += " - {}".format(self.author_uri)
        if self.show_all_details:
            if self.template:
                info += "\nTemplate: {}".format(self.template)
            if self.license:
                info += "\nLicense: {}".format(self.license)
            if self.license_uri:
                info += "\nLicense URI: {}".format(self.license_uri)
            if self.tags:
                info += "\nTags: {}".format(self.tags)
            if self.text_domain:
                info += "\nDomain: {}".format(self.text_domain)
            if self.parents:
                info += "\nParent Theme(s): {}".format(
                    ", ".join([p.slug for p in self.parents])
                )

        return [info]

    def get_infos(self):
        if super().get_infos():
            return ["Theme: {}".format(super().get_infos()[0])]
        else:
            return []

    def get_warnings(self):
        """Return theme warnings"""
        return [
            "{}{}".format(
                "Theme: " if "Vulnerability" not in warning.splitlines()[0] else "",
                warning,
            )
            for warning in super().get_warnings()
        ]

    def get_name(self):
        return "Theme: {}".format(self.slug)



import json
import re
from string import Template


def format_results(results, format, warnings=True, infos=True, nocolor=False):
    """
    Format the results dict into a "html", "cli" or "json" string.

    - results: resutlts dict objject.
    - format: in `"html"`, `"cli"` or `"json"`
    """
    if format == "json":
        return json.dumps(dict(results), indent=4)
    else:
        return build_message(
            dict(results),
            format=format,
            warnings=warnings,
            infos=infos,
            nocolor=nocolor,
        )


def build_message(results, warnings=True, infos=True, format="cli", nocolor=False):
    """Build mail message text base on report and warnngs and info switch"""
    message = ""
    if results["error"]:
        message += "An error occurred.\n".replace(
            "\n", "<br/>\n" if format == "html" else "\n"
        )
    elif results["alerts"]:
        message += "Vulnerabilities have been detected by WPScan.\n".replace(
            "\n", "<br/>\n" if format == "html" else "\n"
        )
    elif results["warnings"]:
        message += "Issues have been detected by WPScan.\n".replace(
            "\n", "<br/>\n" if format == "html" else "\n"
        )

    if results["summary"] and results["summary"]["line"]:
        summary = []

        if format == "cli":
            if results["summary"]["table"]:
                summary.append(
                    format_summary_ascii_table(
                        results["summary"]["table"],
                        results["summary"]["line"],
                        nocolor=nocolor,
                    )
                )
            else:
                summary.append(results["summary"]["line"])
        elif format == "html":
            if results["summary"]["table"]:
                summary.append(
                    format_summary_html(
                        results["summary"]["table"],
                        results["summary"]["line"],
                        nocolor=nocolor,
                    )
                )
            else:
                summary.append(results["summary"]["line"])
        else:
            raise ValueError("format can only be 'cli' or 'html'")

        message += format_issues(
            "Summary", summary, format=format, apply_br_tab_replace_on_issues=False
        )

    if results["error"]:
        message += "\n".replace("\n", "<br/>\n" if format == "html" else "\n")
        message += format_issues("Error", [results["error"]], format=format)

    if results["alerts"]:
        message += "\n".replace("\n", "<br/>\n" if format == "html" else "\n")
        message += format_issues("Alerts", results["alerts"], format=format)

    if results["warnings"] and warnings:
        message += "\n".replace("\n", "<br/>\n" if format == "html" else "\n")
        message += format_issues("Warnings", results["warnings"], format=format)

    if results["infos"] and infos:
        message += "\n".replace("\n", "<br/>\n" if format == "html" else "\n")
        message += format_issues("Informations", results["infos"], format=format)

    if format == "html":
        message = "<div>" + message + "</div>"

    return message


def format_issues(title, issues, format="cli", apply_br_tab_replace_on_issues=True):
    """Format one block of issues to text with the title"""
    message = ""
    if issues:
        if format == "html":
            message += (
                "<br/>\n&nbsp;&nbsp;&nbsp;&nbsp;%s<br/>\n&nbsp;&nbsp;&nbsp;&nbsp;%s<br/>\n<br/>\n"
                % (title, "-" * len(title))
            )
            message += "<br/>\n<br/>\n".join(
                [
                    replace(issue, {"\n": "<br/>\n", "\t": "&nbsp;&nbsp;&nbsp;&nbsp;"})
                    if format == "html" and apply_br_tab_replace_on_issues
                    else issue
                    for issue in issues
                ]
            )
        else:
            message += "\n\t%s\n\t%s\n\n" % (title, "-" * len(title)) + "\n\n".join(
                issues
            )
    return message


def get_table_cell_color(col, val, ansi=False):
    color = ""
    if col == "Version State":
        if ansi:
            color = (
                "yellow"
                if "Outdated" in val
                else "green"
                if "Latest" in val
                else "default"
            )
        else:
            color = (
                "#cccc00"
                if "Outdated" in val
                else "#228B22"
                if "Latest" in val
                else "#000000"
            )
    elif col == "Vulnerabilities":
        if ansi:
            color = "red" if val.isnumeric() and int(val) > 0 else "default"
        else:
            color = "#ba0000" if val.isnumeric() and int(val) > 0 else "#000000"
    elif col == "Status":
        if ansi:
            color = (
                "red"
                if "Alert" in val
                else "yellow"
                if "Warning" in val
                else "green"
                if "Ok" in val
                else "gray"
                if "Unknown" in val
                else "default"
            )
        else:
            color = (
                "#BA0000"
                if "Alert" in val
                else "#cccc00"
                if "Warning" in val
                else "#008000"
                if "Ok" in val
                else "#808080"
                if "Unknown" in val
                else "#000000"
            )
    return color


def format_summary_ascii_table(table, line, nocolor=False):
    """Return a nice string table
    Author: Thierry Husson - Use it as you want but don't blame me.
    """
    try:
        from colors import color, ansilen
    except ImportError:
        nocolor = True

    myDict = table
    colList = ["Component", "Version", "Version State", "Vulnerabilities", "Status"]
    myList = [colList]  # 1st row = header
    for item in myDict:
        myList.append(
            [
                str(
                    item[col]
                    if nocolor and item[col] is not None
                    else color(
                        str(item[col]),
                        fg=get_table_cell_color(col, item[col], ansi=True),
                    )
                    if nocolor == False and item[col] is not None
                    else ""
                )
                for col in colList
            ]
        )
    colSize = [max(map(len if nocolor else ansilen, col)) for col in zip(*myList)]
    formatStr = " | ".join(["{{:<{}s}}".format(i) for i in colSize])
    myList.insert(1, ["-" * i for i in colSize])  # Seperating line
    string = "\n".join(
        formatStr.format(
            *[
                e.ljust(colSize[i] + (0 if nocolor else int(len(e) - ansilen(e))), " ")
                for (i, e) in enumerate(item)
            ]
        )
        for item in myList
    )
    return string + "\n\n" + line


def format_summary_html(table, line, nocolor):
    row_fmt = """      <tr>
        <td>{}</td>
        <td>{}</td>
        <td>{}</td>
        <td>{}</td>
        <td>{}</td>
    </tr>
    """
    table_rows = ""
    for row in table:
        table_rows += row_fmt.format(
            row["Component"],
            row["Version"],
            # Determine version state
            '<span style="color:{color}">{vulns}</span>'.format(
                vulns=row["Version State"],
                color="#000000"
                if nocolor
                else get_table_cell_color("Version State", row["Version State"]),
            ),
            # Determine vulns color
            '<span style="color:{color}">{vulns}</span>'.format(
                vulns=row["Vulnerabilities"],
                color="#000000"
                if nocolor
                else get_table_cell_color("Vulnerabilities", row["Vulnerabilities"]),
            ),
            # Determine status color
            '<b style="color:{color}">{status}</b>'.format(
                status=row["Status"],
                color="#000000"
                if nocolor
                else get_table_cell_color("Status", row["Status"]),
            ),
        )

    return """<table>
        <tr>
            <th>Component</th>
            <th>Version</th>
            <th>Version State</th>
            <th>Vulnerabilities</th>
            <th>Status</th>
        </tr>
    {}
    </table>
    <br/>{}""".format(
        table_rows, line
    )


def replace(text, conditions):
    """Multiple replacements helper method.  Stolen on the web"""
    rep = conditions
    rep = dict((re.escape(k), rep[k]) for k in rep)
    pattern = re.compile("|".join(rep.keys()))
    text = pattern.sub(lambda m: rep[re.escape(m.group(0))], text)
    return text
